Require Import Arith List.

(* If the input contains pairs of numbers, where the second component
   is the smallest multiple of the first component that is larger than
   (n-1), the output contains pairs of numbers, where the second component
   is the smallest multiple of the first component that is larger than n, the
   boolean value indicates whether n was present in the input. *)

Fixpoint update (n:nat) (l:list (nat * nat)) : list (nat * nat) * bool :=
  match l with
    nil => (nil, false)
  | (a,b)::tl => match update n tl with
                   (tl', in_tl) =>
                   let t := beq_nat b n in 
                   ((if t then (a, b + a) else (a, b))::tl', orb  t in_tl)
                 end
  end.

(* For the input n, this function returns the list of all pairs of two numbers
   (a, b), where a is a prime number smaller than or equal to n and b is the
   smallest multiple of a larger than n, this list is in decreasing order of
   prime numbers (first component of pairs). *)
Fixpoint primes (n:nat) : list (nat * nat) :=
  match n with
    S (S 0) => (2,4)::nil
  | S 0 => nil
  | 0 => nil
  | S p => let l := primes p in 
    let (l', t) := update n l in if t then l' else (n,2*n)::l
  end.

(* For the input n, the output is true exactly when n is prime. *)
Definition prime (n:nat) :=
  match primes n with nil => false | (a, _):: _ => beq_nat a n end.

Eval vm_compute in (primes 29, prime 29).
