Inductive bttree (A : Type) : Type :=
  L (a : A) | B (t1 t2 : bttree A) | T (t1 t2 t3 : bttree A).

Inductive blt (A : Type) : bttree A -> bttree A -> Prop :=
| Lt1 : forall x y, blt A x (B A x y)
| Lt2 : forall x y, blt A y (B A x y)
| Lt3 : forall x y z, blt A x (T A x y z)
| Lt4 : forall x y z, blt A y (T A x y z)
| Lt5 : forall x y z, blt A z (T A x y z).

Lemma blt_wf : forall A, well_founded (blt A).
Proof.
intros A x; induction x; apply Acc_intro.
  intros y H; inversion H.
 intros y H; inversion H.
  auto.
 auto.
intros y H; inversion H; auto.
Qed.

Fixpoint size (A : Type) (t : bttree A) :=
  match t with
    L _ => 1
  | B t1 t2 => size A t1 + size A t2 + 1
  | T t1 t2 t3 => size A t1 + size A t2 + size A t3 + 1
  end.

Require Import Relations Wellfounded Arith Omega.

Definition blt' (A : Type) := clos_trans (bttree A) (blt A).

Lemma blt'_wf : forall A, well_founded (blt' A).
intros A.
apply wf_clos_trans.
apply blt_wf.
Qed.

Lemma blt_size : forall A t1 t2, blt A t1 t2 -> size A t1 < size A t2.
Proof.
intros A t1 t2 H; induction H; simpl; omega.
Qed.

Module second_try.

Lemma blt_wf : forall A, well_founded (blt A).
Proof.
intros A.
apply (wf_incl _ _ (ltof (bttree A) (size A))).
 intros x y; unfold ltof; apply blt_size.
apply (well_founded_ltof (bttree A) (size A)).
Qed.