Require Import Arith Omega.

Inductive sqrt_data (n : nat) : Type :=
  csd : forall m, m * m <= n < (m + 1) * (m + 1) -> sqrt_data n.

Lemma th2 : forall n p, n < S p * S p -> p * p <= n ->
   p * p <= n < (p + 1) * (p + 1).
intros n p; replace ((p+1)*(p+1)) with (S p * S p) by ring.
intros; split; assumption.
Qed.

Fixpoint srec n x : n < x * x -> sqrt_data n :=
  match x return n < x * x -> sqrt_data n with
    0 => fun h => False_rec (sqrt_data n) (lt_n_O n h)
  | S p => fun h => match le_lt_dec (p*p) n with
      left h' => csd n p (th2 n p h h')
    | right h' => srec n p h'
    end
  end.

Lemma prf5 : forall n, n < S n * S n.
intros; simpl; generalize (n * S n); intros x; omega.
Qed.

Definition sqrt n : sqrt_data n :=
  srec n (S n) (prf5 n).

Module as_a_proof.

Definition srec : forall n m, n < m * m ->  sqrt_data n.
intros n m; induction m.
SearchPattern (~ _ < 0).
intros abs; elim (lt_n_O _ abs).
destruct (le_lt_dec (m * m) n) as [sqle | nsq].
intros h; apply (csd n m).
replace ((m + 1) * (m + 1)) with (S m * S m) by ring.
split; assumption.
intros _; apply IHm; exact nsq.
Defined.

End as_a_proof.
Print sigT.

SearchPattern (0 <= _).

Lemma th1_drec :
  forall m p q r, p - m = q * S m + r -> m <= p -> S p = S q * S m + r.
intros m p q r hq hl; rewrite (le_plus_minus _ _ hl), hq; ring.
Qed.

SearchPattern (S _ < S _).
Fixpoint drec (n m:nat) : {q : nat & {r | n = q * S m + r /\ r < S m}} :=
  match n return {q : nat & {r | n = q * S m + r /\ r < S m}} with
    0 => existT (fun q => {r | 0 = q * S m + r /\ r < S m}) 0
           (exist (fun r => 0 = r /\ r < S m) 0
              (conj (refl_equal 0) (lt_O_Sn m)))
  | S p => 
    match le_lt_dec m p with
      right h =>
        existT (fun q => {r | S p = q * S m + r /\ r < S m}) 0
          (exist (fun r => S p = r /\ r < S m) (S p)
             (conj (refl_equal _) (lt_n_S _ _ h)))
    | left h' =>
      let (q, sr) := drec (p - m) m in let (r, pf) := sr in
      let (hq, hl) := pf in
      existT (fun q => {r | S p = q * S m + r /\ r < S m}) (S q)
        (exist (fun r => S p = S q * S m + r /\ r < S m) r (conj (th1_drec _ _ _ _ hq h') hl))
    end
  end.

Definition div_nat n m (h : m <> 0) : {q : nat & {r | n = q * m + r /\ r < m}}
  :=
match m return m <> 0 -> {q : nat & {r | n = q * m + r /\ r < m}} with
  0 => fun h : 0 <> 0 => False_rec _ (h (refl_equal 0))
| S m => fun _ => drec n m
end h.

Notation "n / m" := (div_nat n m (beq_nat_false m 0 (refl_equal false))).


Eval lazy in let (q, sr) := 49 / 10 in let (r, _) := sr in (q, r).

Definition div4 n := div_nat n 4 (beq_nat_false 4 0 (refl_equal false)).

Inductive div_data (n m:nat): Type :=
  cdd : forall q r, n = q * m + r -> r < m -> div_data n m.

Lemma ddth1 : forall n m, m <> 0 -> m <= n -> n - m < n.
intros; omega.
Qed.

Lemma ddth2 : forall n m q r, m <= n -> n - m = q * m + r -> n = S q * m + r.
intros n m q r mlen qp; rewrite (le_plus_minus m n mlen), qp; ring.
Qed.

Definition dd : forall n m, m <> 0 -> div_data n m :=
  Fix lt_wf (fun n, forall m, m <> 0 -> div_data n m)
   (fun n f m h =>
      match le_lt_dec m n with
        left mlen =>
          let (q, r, qp, rp) := f (n - m) (ddth1 _ _ h mlen) m h in
            cdd n m (S q) r (ddth2 _ _ _ _ mlen qp) rp
      | right nltm =>
          cdd n m 0 n (refl_equal _) nltm
      end).
Check le_plus_minus.

Definition dd : forall n m, m <> 0 -> div_data n m.
apply (Fix lt_wf (fun n => forall m, m <> 0 -> div_data n m)).
intros n f m mn0.
destruct (le_lt_dec m n) as [mlen | nltm].
  assert (mnq : m + (n - m) = n) by (rewrite <- le_plus_minus; auto).
  assert (mnlt : n - m < n) by omega.
  destruct (f _ mnlt m mn0) as [q r qp rp].
  apply (cdd n m (S q) r);[ | exact rp].
  rewrite <-mnq, qp; ring. 
apply (cdd n m 0 n (refl_equal _) nltm).

Definition sqrt' n : {v : nat & { r | n = v*v + r /\ r <= 2*v}}.
apply (Fix lt_wf); intros n f.
destruct (le_lt_dec 1 n).
  destruct (n / 4) as [q [rem [qp remp]]].
  assert (qlt : q < n) by omega.
  destruct (f _ qlt) as [v [r [vp rp]]].
  destruct (le_lt_dec (4 * v + 1) (4 * r + rem)) as [lrg | sml].
    exists (2 * v + 1); exists ((4*r + rem) - (4 * v + 1)).
    split.
      match goal with |- _ = ?a =>
       replace a with (4 * v * v + ((4 * v + 1) + (4 * r + rem - (4 * v + 1))))
       by ring
      end.
      rewrite <- le_plus_minus, qp, vp; [ring | assumption].
    omega.
  exists (2 * v); exists (4 * r + rem).
  split.
    rewrite qp, vp; ring.
  omega.
  assert (e : n = 0) by omega.
  exists 0; exists 0; split; [rewrite e; reflexivity | omega].
Defined.


